/* 
VS1010 RTC
See datasheet chapter 9.17 for details

The rtc.c source code and rtc.h header are modified versions of the VS1005
codes. The main differences are that the RAM functions of the VS1005 have 
been removed, and the shifter register functions have been added.

Important note: if you wish to store data in the shifter register,
any new RTC function you write must begin with reading the shifter
(ReadShifter()), and end with writing back the value (WriterShifter())
Otherwise the stored data will be lost. Functions provided with this source 
file already include the necessary operations.
*/


#include <vo_stdio.h>
#include <vs1010bRom.h>
#include <vs1010b.h>

// Set control register value and wait for 0. 
// (RTC resets the bit once the action has been performed)
void StartAndWaitRtcReady(register u_int16 rtcCfBits) {
	PERIP(RTC_CF) = rtcCfBits;
	while (PERIP(RTC_CF));
}


void WriteShifter(u_int32 data) {
	// Empty instruction is loaded to stop the RTC from 
	// pushing any other data to the register.
	PERIP(RTC_HIGH) = 0; // No instruction
 	StartAndWaitRtcReady(RTC_CF_IBUSY);

	// Insert data and load to shifter with the data cycly init
	PERIP(RTC_HIGH) = (u_int16)(data >> 16);
	PERIP(RTC_LOW) = (u_int16)data;
	StartAndWaitRtcReady(RTC_CF_DBUSY);
}

u_int32 ReadShifter() {
	u_int32 shifter;

	// Read cycle init loads shifter contents to the data registers.
	StartAndWaitRtcReady(RTC_CF_RDBUSY);
	
	// The contents are saved to the variable and also written back to shifter
	shifter = (((u_int32)PERIP(RTC_HIGH)) << 16) | (u_int32)PERIP(RTC_LOW); 
	WriteShifter(shifter);
	
	return shifter;
}


unsigned long GetRtc(void) {
	// The shifter's value is stored for the duration of the function
	u_int32 shifter = ReadShifter();
	u_int32 rtc;
	
	// Set the read instruction to the data register
	PERIP(RTC_HIGH) = RTC_I_READRTC;
	
	// Instruction cycle init. Loads the read instruction from the 
	// data register. Because the READRTC instruction is used, this
	// will load the counter's value to the shifter register.
 	StartAndWaitRtcReady(RTC_CF_IBUSY);
 	
	// The read flag will move the data from the shifter to the interface
	// data registers, where it can be fetched from.
  	StartAndWaitRtcReady(RTC_CF_RDBUSY);
  	rtc = (((u_int32)PERIP(RTC_HIGH)) << 16) | (u_int32)PERIP(RTC_LOW);

	// The data that was in the shifter earlier is written back.
	WriteShifter(shifter);
  	
  	return rtc;	
}

u_int32 SetRtc(register u_int32 t) {
	// Start with reading the shifter
	u_int32 shifter = ReadShifter();

	// Set LOADTRC instruction and then the instruction cycle flag
  	PERIP(RTC_HIGH) = RTC_I_LOADRTC;
	StartAndWaitRtcReady(RTC_CF_IBUSY);
	
	// Set the data-to-be-written to the interface registers, and 
	// then load it to the shifter by setting the data cycle flag
	PERIP(RTC_HIGH) = (u_int16)(t >> 16);
	PERIP(RTC_LOW) = (u_int16)t;
	StartAndWaitRtcReady(RTC_CF_DBUSY);
 
	// With the load and reset instruction, a separate exec bit must be
	// set to 1 for the instruction to execute. This must be held for 
	// at least 1 second, and then be resetted manually.
	PERIP(RTC_CF) = RTC_CF_EXEC;
	printf("Delay starts \n");
	DelayL(10000000);
	printf("Delay ends \n");
	PERIP(RTC_CF) = 0;
	
	// End with writing back the shifter data.
	WriteShifter(shifter);

	return t;
}

// Returns the 8-bit divider register value
u_int16 GetRtc128(void) {
	// Again, start with reading the shifter
	u_int32 shifter = ReadShifter();
  	u_int32 rtc128;

	// Set instruction and the control bit
	PERIP(RTC_HIGH) = RTC_I_DIV128;
	StartAndWaitRtcReady(RTC_CF_IBUSY);
	
	// Load result from the buffer to the interface register
	StartAndWaitRtcReady(RTC_CF_RDBUSY);
	rtc128 = (PERIP(RTC_HIGH) >> 9) ^ 64;
	
	// And end with writing back the shifter data
	WriteShifter(shifter);

	return rtc128;
}
